@@ -1,31 +1,9 @@
 import math
-import multiprocessing
-import os
 import sys
 import numpy as np
-import pyBigWig
+import os
 
 def setGlobalVariables(args):
-	### input bigwig files
-	setInputFiles(args.ctrlbw, args.expbw)
-	setOutputDirectory(args.o)
-	setBiasFiles(args)
-	setFragLen(args.l)
-	setAnlaysisRegion(args.r, args.bl)
-	setFilterCriteria(args.mi)
-	setBinSize(args.binSize)
-	setNumProcess(args.p)
-	setNormalization(args.norm, args.generateNormBW)
-
-
-def setInputFiles(ctrlbwFiles, expbwFiles):
-	global CTRLBW_NAMES
-	global EXPBW_NAMES
-
-	global CTRLBW_NUM
-	global EXPBW_NUM
-	global SAMPLE_NUM
-
 	global COEFCTRL
 	global COEFEXP
 	global COEFCTRL_HIGHRC
@@ -33,34 +11,24 @@ def setInputFiles(ctrlbwFiles, expbwFiles):
 
 	global HIGHRC
 
+	global I_GENERATE_NORM_BW
+	global I_NORM
+	global MIN_FRAG_FILTER_VALUE
 
-	CTRLBW_NUM = len(ctrlbwFiles)
-	EXPBW_NUM = len(expbwFiles)
-	SAMPLE_NUM = CTRLBW_NUM + EXPBW_NUM
+	global FRAGLEN
+	global BINSIZE
 
-	CTRLBW_NAMES = [0] * CTRLBW_NUM
-	for i in range(CTRLBW_NUM):
-		CTRLBW_NAMES[i] = ctrlbwFiles[i]
+	### input bigwig files
+	setBiasFiles(args)
+	FRAGLEN = args.l
+	BINSIZE = args.binSize
 
-	EXPBW_NAMES = [0] * EXPBW_NUM
-	for i in range(EXPBW_NUM):
-		EXPBW_NAMES[i] = expbwFiles[i]
+	sampleNum = len(args.ctrlbw) + len(args.expbw)
 
-
-def setOutputDirectory(outputDir):
-	global OUTPUT_DIR
-
-	if outputDir is None:
-		outputDir = os.getcwd() + "/CRADLE_correctionResult"
-
-	if outputDir[-1] == "/":
-		outputDir = outputDir[:-1]
-
-	OUTPUT_DIR = outputDir
-
-	dirExist = os.path.isdir(OUTPUT_DIR)
-	if not dirExist:
-		os.makedirs(OUTPUT_DIR)
+	I_GENERATE_NORM_BW, I_NORM = setNormalization(args.norm, args.generateNormBW)
+	MIN_FRAG_FILTER_VALUE = setFilterCriteria(args.mi, sampleNum)
+	seed = setRngSeed(args.rngSeed)
+	writeRngSeed(seed, args.o)
 
 
 def setBiasFiles(args):
@@ -69,7 +37,7 @@ def setBiasFiles(args):
 	global MAP
 	global GQUAD
 	global COVARI_NUM
-	global FA
+	global GENOME
 	global COVARI_ORDER
 
 	SHEAR = 0
@@ -77,7 +45,7 @@ def setBiasFiles(args):
 	MAP = 0
 	GQUAD = 0
 
-	FA = args.faFile
+	GENOME = args.genome
 
 	COVARI_NUM = 0
 
@@ -690,256 +658,35 @@ def setBiasFiles(args):
 			GQAUDFILE[i] = args.gquadFile[i]
 		GQAUD_MAX = args.gquadMax
 
-def setFragLen(fragLen):
-	global FRAGLEN
-	FRAGLEN = int(fragLen)
-
-def setAnlaysisRegion(region, bl):
-	global REGION
-
-	REGION = []
-	inputFilename = region
-	inputStream = open(inputFilename)
-	inputFileContents = inputStream.readlines()
-
-	for i in range(len(inputFileContents)):
-		temp = inputFileContents[i].split()
-		temp[1] = int(temp[1])
-		temp[2] = int(temp[2])
-		REGION.append(temp)
-	inputStream.close()
-
-	if len(REGION) > 1:
-		REGION = np.array(REGION)
-		REGION = REGION[np.lexsort(( REGION[:,1].astype(int), REGION[:,0])  ) ]
-		REGION = REGION.tolist()
-
-		regionMerged = []
-
-		pos = 0
-		pastChromo = REGION[pos][0]
-		pastStart = int(REGION[pos][1])
-		pastEnd = int(REGION[pos][2])
-		regionMerged.append([ pastChromo, pastStart, pastEnd])
-		resultIdx = 0
-
-		pos = 1
-		while pos < len(REGION):
-			currChromo = REGION[pos][0]
-			currStart = int(REGION[pos][1])
-			currEnd = int(REGION[pos][2])
-
-			if (currChromo == pastChromo) and (currStart >= pastStart) and (currStart <= pastEnd):
-				maxEnd = np.max([currEnd, pastEnd])
-				regionMerged[resultIdx][2] = maxEnd
-				pos = pos + 1
-				pastChromo = currChromo
-				pastStart = currStart
-				pastEnd = maxEnd
-			else:
-				regionMerged.append([currChromo, currStart, currEnd])
-				resultIdx = resultIdx + 1
-				pos = pos + 1
-				pastChromo = currChromo
-				pastStart = currStart
-				pastEnd = currEnd
-
-		REGION = regionMerged
-
-	if bl is not None:  ### REMOVE BLACKLIST REGIONS FROM 'REGION'
-		blRegionTemp = []
-		inputStream = open(bl)
-		inputFile = inputStream.readlines()
-		for i in range(len(inputFile)):
-			temp = inputFile[i].split()
-			temp[1] = int(temp[1])
-			temp[2] = int(temp[2])
-			blRegionTemp.append(temp)
-
-		## merge overlapping blacklist regions
-		if len(blRegionTemp) == 1:
-			blRegion = blRegionTemp
-			blRegion = np.array(blRegion)
-		else:
-			blRegionTemp = np.array(blRegionTemp)
-			blRegionTemp = blRegionTemp[np.lexsort( ( blRegionTemp[:,1].astype(int), blRegionTemp[:,0] ) )]
-			blRegionTemp = blRegionTemp.tolist()
-
-			blRegion = []
-			pos = 0
-			pastChromo = blRegionTemp[pos][0]
-			pastStart = int(blRegionTemp[pos][1])
-			pastEnd = int(blRegionTemp[pos][2])
-			blRegion.append([pastChromo, pastStart, pastEnd])
-			resultIdx = 0
-
-			pos = 1
-			while pos < len(blRegionTemp):
-				currChromo = blRegionTemp[pos][0]
-				currStart = int(blRegionTemp[pos][1])
-				currEnd = int(blRegionTemp[pos][2])
-
-				if (currChromo == pastChromo) and (currStart >= pastStart) and (currStart <= pastEnd):
-					blRegion[resultIdx][2] = currEnd
-					pos = pos + 1
-					pastChromo = currChromo
-					pastStart = currStart
-					pastEnd = currEnd
-				else:
-					blRegion.append([currChromo, currStart, currEnd])
-					resultIdx = resultIdx + 1
-					pos = pos + 1
-					pastChromo = currChromo
-					pastStart = currStart
-					pastEnd = currEnd
-			blRegion = np.array(blRegion)
-
-		regionWoBL = []
-		for region in REGION:
-			regionChromo = region[0]
-			regionStart = int(region[1])
-			regionEnd = int(region[2])
-
-			overlappedBl = []
-			## overlap Case 1 : A blacklist region completely covers the region.
-			idx = np.where(
-				(blRegion[:,0] == regionChromo) &
-				(blRegion[:,1].astype(int) <= regionStart) &
-				(blRegion[:,2].astype(int) >= regionEnd)
-				)[0]
-			if len(idx) > 0:
-				continue
-
-			## overlap Case 2
-			idx = np.where(
-				(blRegion[:,0] == regionChromo) &
-				(blRegion[:,2].astype(int) > regionStart) &
-				(blRegion[:,2].astype(int) <= regionEnd)
-				)[0]
-			if len(idx) > 0:
-				overlappedBl.extend( blRegion[idx].tolist() )
-
-			## overlap Case 3
-			idx = np.where(
-				(blRegion[:,0] == regionChromo) &
-				(blRegion[:,1].astype(int) >= regionStart) &
-				(blRegion[:,1].astype(int) < regionEnd)
-				)[0]
-			if len(idx) > 0:
-				overlappedBl.extend( blRegion[idx].tolist() )
-
-			if len(overlappedBl) == 0:
-				regionWoBL.append(region)
-				continue
-
-			overlappedBl = np.array(overlappedBl)
-			overlappedBl = overlappedBl[overlappedBl[:,1].astype(int).argsort()]
-			overlappedBl = np.unique(overlappedBl, axis=0)
-			overlappedBl = overlappedBl[overlappedBl[:,1].astype(int).argsort()]
-
-			currStart = regionStart
-			for pos in range(len(overlappedBl)):
-				blStart = int(overlappedBl[pos][1])
-				blEnd = int(overlappedBl[pos][2])
-
-				if blStart <= regionStart:
-					currStart = blEnd
-				else:
-					if currStart == blStart:
-						currStart = blEnd
-						continue
-
-					regionWoBL.append([ regionChromo, currStart, blStart ])
-					currStart = blEnd
-
-				if (pos == (len(overlappedBl)-1)) and (blEnd < regionEnd):
-					if blEnd == regionEnd:
-						break
-					regionWoBL.append([ regionChromo, blEnd, regionEnd ])
-
-		REGION = regionWoBL
-
-	# check if all chromosomes in the REGION in bigwig files
-	bw = pyBigWig.open(CTRLBW_NAMES[0])
-	regionFinal = []
-	for regionIdx in range(len(REGION)):
-		chromo = REGION[regionIdx][0]
-		start = int(REGION[regionIdx][1])
-		end = int(REGION[regionIdx][2])
-
-		chromoLen = bw.chroms(chromo)
-		if chromoLen is None:
-			continue
-		if end > chromoLen:
-			REGION[regionIdx][2] = chromoLen
-			if chromoLen <= start:
-				continue
-		regionFinal.append([chromo, start, end])
-	bw.close()
-
-	REGION = regionFinal
-
-
-def setFilterCriteria(minFrag):
-	global FILTERVALUE
 
+def setFilterCriteria(minFrag, sampleNum):
 	if minFrag is None:
-		FILTERVALUE = SAMPLE_NUM
+		return sampleNum
 	else:
-		FILTERVALUE = int(minFrag)
-
-
-def setScaler(scalerResult):
-	global CTRLSCALER
-	global EXPSCALER
-
-	CTRLSCALER = [0] * CTRLBW_NUM
-	EXPSCALER = [0] * EXPBW_NUM
-	CTRLSCALER[0] = 1
-
-	for i in range(1, CTRLBW_NUM):
-		CTRLSCALER[i] = scalerResult[i-1]
-	for i in range(EXPBW_NUM):
-		EXPSCALER[i] = scalerResult[i+CTRLBW_NUM-1]
-
-
-def setBinSize(binSize):
-	global BINSIZE
-
-	BINSIZE = int(binSize)
-
-
-def setNumProcess(numProcess):
-	global NUMPROCESS
-
-	systemCPUs = int(multiprocessing.cpu_count())
-
-	if numProcess is None:
-		NUMPROCESS = int(systemCPUs / 2.0 )
-		if NUMPROCESS < 1:
-			NUMPROCESS = 1
-	else:
-		NUMPROCESS = int(numProcess)
-
-	if NUMPROCESS > systemCPUs:
-		print("ERROR: You specified too many cpus! (-p). Running with the maximum cpus in the system")
-		NUMPROCESS = systemCPUs
+		return minFrag
 
 
 def setNormalization(norm, generateNormBW):
-	global I_NORM
-	global I_GENERATE_NormBW
+	norm = not norm.lower() == 'false'
+	generateNormBW = not generateNormBW.lower() == 'false'
 
-	if norm.lower() == 'false':
-		I_NORM = False
-	else:
-		I_NORM = True
-
-	if generateNormBW.lower() == 'false':
-		I_GENERATE_NormBW = False
-	else:
-		I_GENERATE_NormBW = True
-
-	if (not I_NORM) and I_GENERATE_NormBW:
-		print("ERROR: I_NOMR should be 'True' if I_GENERATE_NormBW is 'True'")
+	if (not norm) and generateNormBW:
+		print("ERROR: I_NOMR should be 'True' if I_GENERATE_NORM_BW is 'True'")
 		sys.exit()
+
+	return generateNormBW, norm
+
+
+def setRngSeed(seed):
+	if seed is None:
+		seed = np.random.randint(0, 2**32 - 1)
+
+	np.random.seed(seed)
+	print(f"RNG Seed: {seed}")
+	return seed
+
+
+def writeRngSeed(seed, outputDir):
+	seedFileName = os.path.join(outputDir, "rngseed.txt")
+	with open(seedFileName, "w") as seedFile:
+		seedFile.write(f"{seed}\n")
